#include "imagehandler.h"

#include <QFile>
#include <QFileDialog>

#include <QtNetwork>
#include <Enginio/enginioclient.h>
#include <Enginio/enginioreply.h>

ImageHandler::ImageHandler(QWidget *parent) :
    QWidget(parent)
{
    QByteArray backendId("550aa0cfc8e85c1504026a9e");
    _client = new EnginioClient(this);
    _client->setBackendId(backendId);
}

QString ImageHandler::openImage(){
    QString inputImage = QFileDialog::getOpenFileName(this,
                                                      tr("Open File"),
                                                      QDir::currentPath());
    return inputImage;
}

void ImageHandler::saveImage(QString fileID, QString name)
{
    _filePath = name;
    QJsonObject object;
    object["id"] = fileID;
    const EnginioReply *replyDownload = _client->downloadUrl(object);
    connect(replyDownload, SIGNAL(finished(EnginioReply*)), this, SLOT(onReplyFinished(EnginioReply*)));
}

void ImageHandler::onReplyFinished(EnginioReply *enginioReply)
{
    QString url = enginioReply->data().value("expiringUrl").toString();
    QNetworkRequest request(url);
    _reply = _client->networkManager()->get(request);
    connect(_reply, SIGNAL(finished()), this, SLOT(onDownloadFinished()));
}

void ImageHandler::onDownloadFinished()
{
    QByteArray imageData = _reply->readAll();
    QImage *image = new QImage();
    image->loadFromData(imageData);
    image->save(_filePath);
    emit imageSaved();

    _reply->deleteLater();
    _reply = 0;
}
