import QtQuick 2.0
import QtQuick.LocalStorage 2.0
import QtQuick.Dialogs 1.1
import ImageLoader 1.0
import Enginio 1.0
import "masterSurgeon.js" as JS

Image{
    id: screen
    height: 800
    width: 1280
    source: "/images/images/background.png"

    property bool ready
    property var db
    property bool noError: true

    ImageLoader{
        id: imageLoader
    }
    EnginioClient{
        id: client
        backendId: "550aa0cfc8e85c1504026a9e"
        onFinished: {
            db = LocalStorage.openDatabaseSync("LocalDB", "1.0", "Local Database", 1000000);
            if(!ready) setUp();
            if(noError){
                var data = reply.data.results
                for(var i = 0; i < data.length; i++){
                    if(data[i].gamemode === "tb" && compare(JS.levels_tb, data[i]))insertLevels_tb(data[i].id, data[i].solution, data[i].name, data[i].mm);
                    if(data[i].gamemode === "lm" && compare(JS.levels_lm, data[i]))insertLevels_lm(data[i].id, data[i].landmarkname, data[i].solution, data[i].name, data[i].mm);
                }
                imageLoader.saveImages();
                state.stateUrl = Qt.resolvedUrl("masterSurgeon.qml")
            }
        }
        onError: {
            noError = false;
            errorDialog.visible = true;
        }
        Component.onCompleted: query({
                                         "objectType": "objects.level",
                                         "include": {"file": {}},
                                         "query": { "file": { "$ne": null } }
                                     });
    }
    Column{
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter
        Image{
            id: logo
            height: screen.width/3
            width: screen.width/3
            source: "/images/images/loading.png"
        }
        Image{
            id: animation
            anchors.horizontalCenter: parent.horizontalCenter
            width: screen.height/10
            height: screen.height/10
            source: "/images/images/loadingIcon.png"

            RotationAnimation on rotation {
                loops: Animation.Infinite
                from: 0
                to: 360
            }
        }
    }
    function setUp(){
        db.transaction(
                    function(tx) {

                        JS.levels_tb = [];
                        tx.executeSql('CREATE TABLE IF NOT EXISTS Levels_tb(id TEXT, solution TEXT, localPath TEXT, mm REAL)');
                        var tempTB = tx.executeSql('SELECT * FROM Levels_tb');
                        for(var i = 0; i < tempTB.rows.length; i++) {
                            JS.levels_tb.push({id: tempTB.rows.item(i).id, solution: JSON.parse(tempTB.rows.item(i).solution), localPath: tempTB.rows.item(i).localPath, mm: tempTB.rows.item(i).mm})
                        }

                        JS.levels_lm = [];
                        tx.executeSql('CREATE TABLE IF NOT EXISTS Levels_lm(id TEXT, landmarkname TEXT, solution TEXT, localPath TEXT, mm REAL)')
                        var tempLM = tx.executeSql('SELECT * FROM Levels_lm');
                        for(var i = 0; i < tempLM.rows.length; i++) {
                            JS.levels_lm.push({id: tempLM.rows.item(i).id, landmarkname: tempLM.rows.item(i).landmarkname, solution: JSON.parse(tempLM.rows.item(i).solution), localPath: tempLM.rows.item(i).localPath, mm: tempLM.rows.item(i).mm})
                        }

                        JS.scores = [];
                        tx.executeSql('CREATE TABLE IF NOT EXISTS Scores(id TEXT, score INT, time INT)');
                        var tempScore = tx.executeSql('SELECT * FROM Scores');
                        for(var i = 0; i < tempScore.rows.length; i++) {
                            JS.scores.push({id: tempScore.rows.item(i).id, score: tempScore.rows.item(i).score, time: tempScore.rows.item(i).time})
                        }
                    })
        ready = true;
    }
    function compare(list, level){
        for(var j = 0; j < list.length; j++){
            if(level.id === list[j].id){
                return false;
            }
        }
        imageLoader.prepareImage(level.file.id, level.name);
        return true;
    }
    function insertLevels_tb(id, solution, localPath, mm) {
        db.transaction(
                    function(tx) {
                        tx.executeSql('INSERT INTO Levels_tb VALUES(?, ?, ?, ?)', [ id, JSON.stringify(solution), localPath, mm]);
                        JS.levels_tb.push({id: id, solution: solution, localPath: localPath, mm: mm});
                    })
    }
    function insertLevels_lm(id, landmarkname, solution, localPath, mm) {
        db.transaction(
                    function(tx) {
                        tx.executeSql('INSERT INTO Levels_lm VALUES(?, ?, ?, ?, ?)', [ id, landmarkname, JSON.stringify(solution), localPath, mm]);
                        JS.levels_lm.push({id: id, landmarkname: landmarkname, solution: solution, localPath: localPath, mm: mm});
                    })
    }
    MessageDialog {
        id: errorDialog
        text: "Could not connect to the cloud database.\n\nIf this is the first time you are running this application, an internet connection is required to download all exercises. You may continue to play in offline mode, but you will not recive any updates until you start this application with an internett connection."
        onAccepted: offlineMode();
        Component.onCompleted: visible = false;
    }

    Item {
        id: state
        visible: false
        property url stateUrl
        onStateUrlChanged: visible = (stateUrl == '') ? false : true;
        anchors.fill: parent

        Loader{
            focus: true
            source: state.stateUrl
            anchors.fill: parent
        }
    }
    function offlineMode(){
        state.stateUrl = Qt.resolvedUrl("masterSurgeon.qml");
    }
}
