import QtQuick 2.0
import QtQuick.Dialogs 1.1
import QtQuick.Window 2.2
import QtQuick.LocalStorage 2.0
import ImageLoader 1.0
import Enginio 1.0
import "landmarks.js" as JS
import "masterSurgeon.js" as Input
import "../shared"
import "../images"

Rectangle {
    id: screen
    color: "black"

    property int buttonSize: screen.height/10
    property int seconds
    property int counter
    property int result
    property int xPos
    property int yPos

    ImageLoader{
        id: loader
    }

    WorkerScript {
        id: myWorker
        source: "landmarks.js"

        onMessage: {
            result =  messageObject.result;
            var string;
            var highscore;
            var score

            if(Input.level.mm === 0) score = Math.round(result/5);
            else score = result/Input.level.mm;

            if(result < 0){
                string = "Did you even draw? Try again!"
            }
            else if(score == 0){
                highscore = saveScores(3);
                resultWindow.s3 = "/images/images/syringeActive.png"
                resultWindow.s2 = "/images/images/syringeActive.png"
                resultWindow.s1 = "/images/images/syringeActive.png"
                string = "Spot on!"

            }
            else if(score < 5){
                highscore = saveScores(2);
                resultWindow.s2 = "/images/images/syringeActive.png"
                resultWindow.s1 = "/images/images/syringeActive.png"
                string = "Well done!"
            }
            else{
                highscore  = saveScores(1);
                resultWindow.s1 = "/images/images/syringeActive.png"
                string = "Keep practicing and\nyou will get better!"
            }
            string = string.concat("\n\nDifference in pixels: " + result);
            if(Input.level.mm !== 0) string = string.concat("\nDifference in mm: " + score.toFixed(1));
            string = string.concat("\n\nTime: " + seconds + " seconds!");
            resultWindow.index = "Case " + Input.index;
            resultWindow.result = string
            resultWindow.visible = true
            hideButton.visible = true;
            animation.visible = false;
        }
    }

    Flickable {
        id: flick
        width: Screen.width
        height: Screen.height
        anchors.fill: parent
        contentWidth: image.width
        contentHeight: image.height + buttonSize

        Image{
            id:image
            source: loader.localPath(Input.level.localPath)

            Canvas {
                id:canvas
                anchors.fill: parent
            }
        }
        Image {
            id: marker
            width: 100
            height: 100
            source: "/images/images/landMarker.png"
            visible: false
        }
        Image {
            id: goldMarker
            width: 100
            height: 100
            source: "/images/images/landMarkerGold.png"
            visible: false
        }

        MouseArea{
            anchors.fill: parent
            onClicked:{
                if(markerButton.active){
                    marker.visible = true;
                    marker.x = Math.floor(mouseX) - (marker.width/2);
                    marker.y = Math.floor(mouseY) - marker.height;
                }
            }
            onPositionChanged:{
                if(markerButton.active){
                    marker.visible = true;
                    marker.x = Math.floor(mouseX) - (marker.width/2);
                    marker.y = Math.floor(mouseY) - marker.height;
                }
            }
            onReleased:{
                if(markerButton.active){
                    marker.visible = true;
                    xPos = Math.floor(mouseX);
                    yPos = Math.floor(mouseY);
                }
            }
        }
    }

    MessageDialog {
        id: exitDialog
        text: "Do you want to exit to main menu?"
        standardButtons: StandardButton.No | StandardButton.Yes
        onYes: state.stateUrl = Qt.resolvedUrl("masterSurgeon.qml");
        onNo: {
            gameTimer.start();
            exitButton.active = false;
        }
        Component.onCompleted: visible = false
    }
    MessageDialog {
        id: sendDialog
        text: "Are you finished?"
        standardButtons: StandardButton.No | StandardButton.Yes
        onYes:{
            markerButton.active = false;
            flick.interactive = false;
            exitButton.visible = false;
            layout.visible = false;
            timerText.visible = false;
            animation.visible = true;
            myWorker.sendMessage({solution: Input.level.solution, x: xPos, y: yPos});
            flick.contentX = Input.level.solution[0].x - (screen.width/2);
            flick.contentY = Input.level.solution[0].y - (screen.height/2);
            if(Input.level.solution.length === 1){
                goldMarker.x = Input.level.solution[0].x - (marker.width/2);
                goldMarker.y = Input.level.solution[0].y - marker.height;
                goldMarker.visible = true;
            }
            else JS.drawSolution(canvas, Input.level.solution)
            canvas.requestPaint();
        }
        onNo:{
            gameTimer.start();
            sendButton.active = false;
        }
        Component.onCompleted: visible = false

    }
    Text {
        id: timerText
        anchors.horizontalCenter: parent.horizontalCenter
        font.pointSize: buttonSize/5
        color: "#92e9ff"

        Timer {
            id: gameTimer
            running: true
            interval: 1000
            repeat: true
            onTriggered:{
                seconds++;
                timerText.text = padding(Math.floor(seconds/60)) + ":" + padding(seconds%60);
            }
        }
    }
    Image {
        id: layout
        width: screen.width
        height: buttonSize + 2
        anchors.left: parent.left
        anchors.bottom: parent.bottom
        source: "/images/images/bar.png"

        Button {
            id: markerButton
            width: buttonSize
            height: buttonSize
            anchors.bottom: parent.bottom
            anchors.left: parent.left
            imgSrc: active ? "/images/images/buttonLandmarkerActive.png" : "/images/images/buttonLandmarker.png"
            onClicked: {
                if(!markerButton.active){
                    markerButton.active = true;
                    flick.interactive = false;
                }
                else {
                    markerButton.active = false;
                    flick.interactive = true;
                }
            }
        }
        Text{
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.verticalCenter: parent.verticalCenter
            font.pointSize: buttonSize/2
            text: "Locate " + Input.level.landmarkname
            color: "#92e9ff"
        }

        Button {
            id: sendButton
            width: buttonSize
            height: buttonSize
            anchors.bottom: parent.bottom
            anchors.right: parent.right
            imgSrc: active ? "/images/images/buttonSendActive.png" : "/images/images/buttonSend.png"
            onClicked: {
                gameTimer.stop();
                sendButton.active = true;
                sendDialog.visible = true;
            }
        }
    }
    Button{
        id: exitButton
        anchors.right: parent.right
        width: screen.width/20
        height: screen.width/20
        imgSrc: active ? "/images/images/settingsButtonActive.png" : "/images/images/settingsButton.png"
        onClicked: {
            gameTimer.stop();
            exitButton.active = true;
            exitDialog.visible = true;
        }
    }
    Image{
        id: animation
        anchors.horizontalCenter: screen.horizontalCenter
        anchors.verticalCenter: screen.verticalCenter
        visible: false;
        width: buttonSize*2
        height: buttonSize*2
        source: "/images/images/loadingIcon.png"

        RotationAnimation on rotation {
            loops: Animation.Infinite
            from: 0
            to: 360
        }
    }
    ResultWindow{
        id: resultWindow
        anchors {
            top: parent.top
            bottom: parent.bottom
            right: parent.horizontalCenter
            left: parent.left
        }
        anchors.margins: buttonSize+2
        visible: false

        Row{
            id: buttons
            spacing: 10
            anchors.horizontalCenter: parent.horizontalCenter
            anchors.bottom: parent.bottom
            anchors.bottomMargin: buttonSize

            Button{
                id: repeatButton
                height: buttonSize;
                width: buttonSize;
                imgSrc: "/images/images/buttonRepeat.png"
                onClicked: state.stateUrl = Qt.resolvedUrl("landmarks.qml")
            }
            Button{
                id: playMoreButton
                height: buttonSize;
                width: buttonSize*3;
                imgSrc: "/images/images/buttonMainMenu.png"
                onClicked: state.stateUrl = Qt.resolvedUrl("loading.qml")
            }
        }
    }
    Button{
        id: hideButton
        anchors.horizontalCenter: resultWindow.horizontalCenter
        y: resultWindow.height + (screen.height - resultWindow.height)/2 - screen.width/40
        width: screen.width/20
        height: screen.width/20
        visible: false;
        imgSrc: active ? "/images/images/buttonEyeBlueActive.png" : "/images/images/buttonEyeBlue.png"
        onClicked: {
            if(hideButton.active){
                resultWindow.visible = true;
                flick.interactive = false;
                hideButton.active = false;
            }
            else{
                resultWindow.visible = false;
                flick.interactive = true;
                hideButton.active = true;
            }
        }
    }
    Item {
        id: state
        visible: false
        property url stateUrl
        onStateUrlChanged: visible = (stateUrl == '') ? false : true;
        anchors.fill: parent

        Loader{
            focus: true
            source: state.stateUrl
            anchors.fill: parent
        }
    }
    function padding(number){
        if(number < 10) return "0" + number;
        return number;
    }
    function saveScores(score){
        var db = LocalStorage.openDatabaseSync("LocalDB", "1.0", "Local Database", 1000000);
        db.transaction(
                    function(tx) {
                        for(var i = 0; i < Input.scores.length; i++){
                            if(Input.level.id === Input.scores[i].id && score > Input.scores[i].score){
                                tx.executeSql('UPDATE Scores SET score = ?, time = ? WHERE id = ?', [score, seconds, Input.level.id]);
                                return true;
                            }
                        }
                        tx.executeSql('INSERT INTO Scores VALUES(?, ?, ?)', [Input.level.id, score, seconds]);
                        return false;
                    })
    }
}
