import QtQuick 2.0
import ImageLoader 1.0
import QtQuick.Window 2.2
import QtQuick.Dialogs 1.2
import Enginio 1.0
import "landmarksGold.js" as JS
import "../shared"
import "../images"

Rectangle {
    id: screen
    color: "black"

    property int buttonSize: screen.height/10
    property bool noError: true
    property int xPos: -1
    property int yPos: -1

    ImageLoader{
        id: loader
    }

    EnginioClient{
        id: enginio
        backendId: "550aa0cfc8e85c1504026a9e"
        onFinished: if(noError){
                        animation.visible = false
                        confirmDialog.visible = true
                    }
        onError: {
            noError = false
            errorDialog.visible = true
        }

    }
    Flickable {
        id: flick
        width: Screen.width
        height: Screen.height
        anchors.fill: parent
        contentWidth: image.width
        contentHeight: image.height + buttonSize

        Image {
            id:image
            Component.onCompleted: {
                openFileButton.active = true
                var filename = loader.openFile()
                if(filename) {
                    image.source = "file:///" + filename
                    JS.setUpCanvas(canvas, flick.width/20)
                }
                openFileButton.active = false
            }

            Canvas {
                id:canvas
                anchors.fill: parent
                onPaint: {
                    if(undoButton.active) JS.remove()
                    else if(brushButton.active) JS.paintBrush()
                }

                MouseArea{
                    anchors.fill: parent
                    onClicked:{
                        if(pointButton.active){
                            JS.clearCanvas(canvas.height, canvas.width)
                            canvas.requestPaint()
                            marker.visible = true
                            marker.x = Math.floor(mouseX) - (marker.width/2)
                            marker.y = Math.floor(mouseY) - marker.height
                        }
                        else if(mmButton.active){
                            offset.getImage().visible = true
                            offset.getImage().x = Math.floor(mouseX) - (offset.getImage().width/2)
                            offset.getImage().y = Math.floor(mouseY) - (offset.getImage().height/2)
                            offset.active = offset.active ? false : true

                        }
                    }
                    onPositionChanged:{
                        if(pointButton.active){
                            marker.visible = true
                            marker.x = Math.floor(mouseX) - (marker.width/2)
                            marker.y = Math.floor(mouseY) - marker.height
                        }
                        else if(brushButton.active || undoButton.active){
                            xPos = -1
                            yPos = -1
                            marker.visible = false
                            JS.setPosition(Math.floor(mouseX), Math.floor(mouseY))
                            canvas.requestPaint()
                        }
                    }
                    onReleased:{
                        if(pointButton.active){
                            marker.visible = true
                            xPos = Math.floor(mouseX)
                            yPos = Math.floor(mouseY)
                        }
                        else JS.resetPosition()
                    }
                }
            }
        }
        Image {
            id: marker
            width: 100
            height: 100
            source: "/images/images/landMarker.png"
            visible: false
        }
        Offset{
            id:offset
            anchors.fill: parent
            imageSize: buttonSize
        }
    }
    MessageDialog {
        id: errorDialog
        text: "Could not connect to the cloud database.\n\nAn internet connection is required to upload new exercises."
        Component.onCompleted: visible = false
    }
    MessageDialog {
        id: exitDialog
        text: "Do you want to exit to main menu?"
        standardButtons: StandardButton.No | StandardButton.Yes
        onYes: state.stateUrl = Qt.resolvedUrl("loading.qml")
        onNo: exitButton.active = false
        Component.onCompleted: visible = false
    }
    MessageDialog {
        id: confirmDialog
        text: "Case saved successfully"
        onAccepted: {
            state.stateUrl = Qt.resolvedUrl("loading.qml")
        }
        Component.onCompleted: visible = false
    }
    MessageDialog {
        id: noSolutionDialog
        text: "No solution detected.\n\nPlease use the point or brush tool to draw a border."
        onAccepted: animation.visible = false
        Component.onCompleted: visible = false
    }
    MessageDialog {
        id: clearDialog
        text: "Do you really want to clear your drawing?"
        standardButtons: StandardButton.No | StandardButton.Yes
        onYes:{
            undoButton.active = false
            brushButton.active = false
            pointButton.active = false
            trashButton.active = false
            mmButton.active = false
            flick.interactive = true
            JS.clearCanvas(canvas.height, canvas.width)
            canvas.requestPaint()
        }
        onNo: trashButton.active = false
        Component.onCompleted: visible = false

    }
    Image {
        width: screen.width
        height: screen.height/10 + 2
        anchors.left: parent.left
        anchors.bottom: parent.bottom
        source: "/images/images/bar.png"

        Row {
            id: layout
            anchors.bottom: parent.bottom
            anchors.left: parent.left
            anchors.right: parent.right

            Button {
                id: pointButton
                width: buttonSize
                height: buttonSize
                anchors.bottom: parent.bottom
                imgSrc: active ? "/images/images/buttonLandmarkerActive.png" : "/images/images/buttonLandmarker.png"
                onClicked: {
                    if(pointButton.active){
                        flick.interactive = true
                        pointButton.active = false
                    }
                    else{
                        flick.interactive = false
                        pointButton.active = true
                        undoButton.active = false
                        mmButton.active = false
                        brushButton.active = false
                    }
                }
            }
            Button {
                id: brushButton
                width: buttonSize
                height: buttonSize
                anchors.bottom: parent.bottom
                imgSrc: active ? "/images/images/buttonBrushActive.png" : "/images/images/buttonBrush.png"
                onClicked: {
                    if(brushButton.active){
                        flick.interactive = true
                        brushButton.active = false
                    }
                    else{
                        flick.interactive = false
                        brushButton.active = true
                        undoButton.active = false
                        mmButton.active = false
                        pointButton.active = false
                    }
                }
            }
            Button {
                id: undoButton
                anchors.bottom: parent.bottom
                width: buttonSize
                height: buttonSize
                imgSrc: active ? "/images/images/buttonUndoActive.png" : "/images/images/buttonUndo.png"
                onClicked: {
                    if(undoButton.active){
                        flick.interactive = true
                        undoButton.active = false
                    }
                    else{
                        flick.interactive = false
                        undoButton.active = true
                        brushButton.active = false
                        mmButton.active = false
                        pointButton.active = false
                    }
                }
            }
            Button {
                id: trashButton
                anchors.bottom: parent.bottom
                width: buttonSize
                height: buttonSize
                imgSrc: active ? "/images/images/buttonTrashActive.png" : "/images/images/buttonTrash.png"
                onClicked: {
                    trashButton.active = true
                    clearDialog.visible = true
                }
            }
            Button{
                id: mmButton
                anchors.bottom: parent.bottom
                width: buttonSize
                height: buttonSize
                imgSrc: active ? "/images/images/buttonMMActive.png" : "/images/images/buttonMM.png"
                onClicked: {
                    if(mmButton.active){
                        Qt.inputMethod.hide()
                        mmButton.active = false
                        offset.visible = false
                        mmLayout.visible = false
                    }
                    else{
                        mmButton.active = true
                        undoButton.active = false
                        pointButton.active = false
                        brushButton.active = false
                        offset.visible = true
                        mmLayout.visible = true
                    }
                }
            }
            Button{
                id: hideButton
                anchors.bottom: parent.bottom
                width: buttonSize
                height: buttonSize
                imgSrc: active ? "/images/images/buttonEyeActive.png" : "/images/images/buttonEye.png"
                onClicked: {
                    if(hideButton.active){
                        canvas.visible = true
                        hideButton.active = false
                    }
                    else{
                        canvas.visible = false
                        hideButton.active = true
                    }
                }
            }
            Row{
                id: mmLayout
                visible: false
                anchors.verticalCenter: parent.verticalCenter

                Text {
                    text: "Offset:"
                    font.pointSize: buttonSize/2
                    anchors.verticalCenter: parent.verticalCenter
                }
                Rectangle{
                    color: "white"
                    border.color: "black"
                    height: (buttonSize/2) + 5
                    width: buttonSize * 2
                    anchors.verticalCenter: parent.verticalCenter

                    TextInput {
                        id: offsetText
                        anchors.right: parent.right
                        anchors.verticalCenter: parent.verticalCenter
                        text: "0"
                        font.pointSize: buttonSize/2
                        Keys.onEnterPressed: {
                            Qt.inputMethod.hide();
                        }
                        Keys.onReturnPressed: {
                            Qt.inputMethod.hide();
                        }
                    }
                    MouseArea{
                        anchors.fill: parent
                        onClicked: {
                            offsetText.focus = false;
                            offsetText.selectAll();
                            offsetText.forceActiveFocus();
                        }
                    }
                }
                Text {
                    text: "mm"
                    font.pointSize: buttonSize/2
                    anchors.verticalCenter: parent.verticalCenter
                }
            }
        }
    }
    Row {
        layoutDirection: "RightToLeft"
        anchors.bottom: parent.bottom
        anchors.left: parent.left
        anchors.right: parent.right

        Button {
            id: sendButton
            anchors.bottom: parent.bottom
            width: buttonSize
            height: buttonSize
            imgSrc: active ? "/images/images/buttonSaveActive.png" : "/images/images/buttonSave.png"
            onClicked: {
                sendButton.active = true
                sendDialog.visible = true
            }
        }
        Button {
            id: openFileButton
            anchors.bottom: parent.bottom
            width: buttonSize
            height: buttonSize
            imgSrc: active ? "/images/images/buttonOpenFileActive.png" : "/images/images/buttonOpenFile.png"
            onClicked: {
                openFileButton.active = true
                var filename = loader.openFile()
                if(filename) {
                    image.source = "file:///" + filename
                    marker.visible = false;
                    xPos = -1
                    yPos = -1
                    JS.setUpCanvas(canvas, flick.width/20)
                }
                openFileButton.active = false
            }
        }
    }

    Button{
        id: exitButton
        anchors.right: parent.right
        width: screen.width/20
        height: screen.width/20
        imgSrc: active ? "/images/images/settingsButtonActive.png" : "/images/images/settingsButton.png"
        onClicked: {
            exitButton.active = true
            exitDialog.visible = true
        }
    }
    Image{
        id: animation
        anchors.horizontalCenter: screen.horizontalCenter
        anchors.verticalCenter: screen.verticalCenter
        visible: false
        width: buttonSize*2
        height: buttonSize*2
        source: "/images/images/loadingIcon.png"

        RotationAnimation on rotation {
            loops: Animation.Infinite
            from: 0
            to: 360
        }
    }
    Dialog{
        id: sendDialog
        width: screen.width/1.5
        height: screen.height/3
        standardButtons: StandardButton.Save | StandardButton.Cancel

        Column {
            spacing: 5
            anchors.fill: parent
            Text {
                text: "When you are are ready to save your solution press save."
            }
            Rectangle{
                color: "white"
                border.color: "black"
                height: landmarkName.height + 2
                width: screen.width/2
                TextInput {
                    id: landmarkName
                    anchors.verticalCenter: parent.verticalCenter
                    text: "Enter the name of this landmark..."
                    Keys.onEnterPressed: {
                        Qt.inputMethod.hide();
                    }
                    Keys.onReturnPressed: {
                        Qt.inputMethod.hide();
                    }
                }
                MouseArea{
                    anchors.fill: parent
                    onClicked: {
                        landmarkName.focus = false;
                        landmarkName.selectAll();
                        landmarkName.forceActiveFocus();
                    }
                }
            }
        }
        onButtonClicked: {
            if (clickedButton  === StandardButton.Save) {
                animation.visible = true

                if(xPos == -1 && yPos == -1 && JS.send().length < 1) noSolutionDialog.visible = true
                else{
                    if(xPos != -1 && yPos != -1)JS.solutionPoint(xPos, yPos)
                    landmarkName.visible = true
                    var os = 0
                    if(parseInt(offsetText.text) !== 0 && offsetText.text.length > 1) os = offset.offset()/parseInt(offsetText.text)
                    var pathParts = image.source.toString().split("/")
                    var fileName = pathParts[pathParts.length - 1]
                    var fileObject = {
                        objectType: "objects.level",
                        name: fileName,
                        gamemode: "lm",
                        landmarkname: landmarkName.text,
                        solution: JS.solution,
                        localPath: image.source,
                        mm: os
                    }
                    var reply = enginio.create(fileObject)
                    reply.finished.connect(function() {
                        var uploadData = {
                            file: { fileName: fileName },
                            targetFileProperty: {
                                objectType: "objects.level",
                                id: reply.data.id,
                                propertyName: "file"
                            },
                        }
                        enginio.uploadFile(uploadData, image.source)
                    })
                    sendButton.active = false
                }
            }
            else sendButton.active = false
            landmarkName.focus = false
        }
    }

    Item {
        id: state
        visible: false
        property url stateUrl
        onStateUrlChanged: visible = (stateUrl == '') ? false : true
        anchors.fill: parent

        Loader{
            focus: true
            source: state.stateUrl
            anchors.fill: parent
        }
    }
}
