import QtQuick 2.2
import Enginio 1.0
import QtQuick.LocalStorage 2.0
import "masterSurgeon.js" as JS
import "../shared"
import "../images"

Image {
    id: screen
    height: 800
    width: 1280
    source: "/images/images/background.png"

    property bool gameMode

    //Purely for testing. SHOULD NOT BE INCLUDED IN FINAL PRODUCT
    Button{
        id: localReset
        imgSrc: "/images/images/buttonLocalResetl.png"
        width: screen.width/20
        height: screen.width/20
        anchors.right: parent.right;
        onClicked:{
            var db = LocalStorage.openDatabaseSync("LocalDB", "1.0", "Local Database", 1000000);
            db.transaction(
                        function(tx) {
                            tx.executeSql('DROP TABLE Levels_tb');
                            tx.executeSql('DROP TABLE Levels_lm');
                            tx.executeSql('DROP TABLE Scores');
                        })
            state.stateUrl = Qt.resolvedUrl("loading.qml");
        }
    }
    Row{
        id: mainMenu
        anchors.horizontalCenter: parent.horizontalCenter
        anchors.verticalCenter: parent.verticalCenter


        Button{
            id: tmButton
            imgSrc: "/images/images/mainMenuTB.png"
            width: screen.width/5
            height: screen.width/5
            onClicked:{
                gameMode = true;
                showLevels(JS.levels_tb);
                imgSrc = "/images/images/mainMenuTB.png"
            }
            onPressedChanged: tmButton.imgSrc = "/images/images/mainMenuTBActive.png"
        }
        Button{
            id: lmButton
            imgSrc: "/images/images/mainMenuLM.png"
            width: screen.width/5
            height: screen.width/5
            onClicked:{
                gameMode = false;
                showLevels(JS.levels_lm);
                imgSrc = "/images/images/mainMenuLM.png"
            }
            onPressedChanged: lmButton.imgSrc = "/images/images/mainMenuLMActive.png"
        }
        Button{
            id: tmButtonGold
            imgSrc: "/images/images/mainMenuTBG.png"
            width: screen.width/5
            height: screen.width/5
            onClicked:{
                state.stateUrl = Qt.resolvedUrl("tumorBorderGold.qml")
                imgSrc = "/images/images/mainMenuTBG.png"
            }
            onPressedChanged: tmButtonGold.imgSrc = "/images/images/mainMenuTBGActive.png"
        }
        Button{
            id: lmButtonGold
            imgSrc: "/images/images/mainMenuLMG.png"
            width: screen.width/5
            height: screen.width/5
            onClicked:{
                state.stateUrl = Qt.resolvedUrl("landmarksGold.qml")
                imgSrc = "/images/images/mainMenuLMG.png"
            }
            onPressedChanged: lmButtonGold.imgSrc = "/images/images/mainMenuLMGActive.png"
        }
    }
    GridView {
        id: levelMenu
        model: levelModel
        delegate: levelDelegate
        anchors.fill: parent
        anchors.margins: screen.width/20
        visible: false
    }
    ListModel {
        id: levelModel
    }
    Component{
        id: levelDelegate

        Level {
            id: level
            number: index + 1;
            onClicked: {
                if(gameMode) JS.level = JS.levels_tb[index]
                else JS.level = JS.levels_lm[index]
                JS.index = index+1;
                startGame();
            }
            Component.onCompleted: {
                for(var i = 0; i < JS.scores.length; i++){
                    if(JS.scores[i].id === id){
                        if(JS.scores[i].score > 0) level.s1 = "/images/images/syringeActive.png"
                        if(JS.scores[i].score > 1) level.s2 = "/images/images/syringeActive.png"
                        if(JS.scores[i].score > 2) level.s3 = "/images/images/syringeActive.png"
                        level.showStars = true;
                    }
                }
            }
        }

    }
    Button{
        id: backButton
        width: screen.width/20
        height: screen.width/20
        anchors.bottom: parent.bottom
        imgSrc: "/images/images/buttonBack.png"
        visible: active ? true : false
        onClicked: {
            active = true;
            backButton.visible = false;
            levelMenu.visible = false;
            mainMenu.visible = true;
            imgSrc = "/images/images/buttonBack.png"
        }
        onPressedChanged: backButton.imgSrc = "/images/images/buttonBackActive.png"

    }
    Item {
        id: state
        visible: false
        property url stateUrl
        onStateUrlChanged: visible = (stateUrl == '') ? false : true;
        anchors.fill: parent

        Loader{
            focus: true
            source: state.stateUrl
            anchors.fill: parent
        }
    }
    function showLevels(array) {
        levelModel.clear();
        for(var i = 0; i < array.length; i++) {
            levelModel.append(array[i]);
        }
        levelMenu.visible = true;
        mainMenu.visible = false;
        backButton.visible = true;
    }
    function startGame(){
        if(gameMode) state.stateUrl = Qt.resolvedUrl("tumorBorder.qml")
        else state.stateUrl = Qt.resolvedUrl("landmarks.qml")
    }
}
