var mouseX = -1;
var mouseY = -1;
var oldX = -1;
var oldY = -1;
var canvasWidth;
var canvasHeight;
var removeSize;
var ctx;
var points = [];

function setUpCanvas(canvas, size){
    ctx = canvas.getContext('2d');
    ctx.fillStyle = "#00ccff";
    canvasWidth = canvas.width;
    canvasHeight = canvas.height;
    removeSize = Math.floor(size/8);
    emptyPoints();
}
function paintLine(){
    if(oldX == -1 && oldY == -1){
        oldX = mouseX;
        oldY = mouseY;
    }
    if(oldX != -1) drawPoints(oldX, oldY, mouseX, mouseY);
    oldX = mouseX;
    oldY = mouseY;
}
function remove(){
    for(var x = 0; x < removeSize; x++){
        for(var y = 0; y < removeSize; y++){
            points[mouseX+x][mouseY+y] = 0;
            points[mouseX-x][mouseY-y] = 0;
            points[mouseX+x][mouseY-y] = 0;
            points[mouseX-x][mouseY+y] = 0;
            ctx.clearRect(mouseX+x, mouseY+y, 1, 1);
            ctx.clearRect(mouseX-x, mouseY-y, 1, 1);
            ctx.clearRect(mouseX+x, mouseY-y, 1, 1);
            ctx.clearRect(mouseX-x, mouseY+y, 1, 1);
        }
    }
}
function setPosition(xNew, yNew){
    mouseX = xNew;
    mouseY = yNew;
}
function resetPosition(){
    oldX = -1;
    oldY = -1;
    mouseX = -1;
    mouseY = -1;
}
function clearCanvas(height, width) {
    ctx.clearRect(0, 0, width, height);
    points.length = 0;
    emptyPoints();
    resetPosition();
}
function emptyPoints(){
    points.length = canvasWidth;
    for (var i = 0; i < canvasWidth; i++) {
        points[i] = new Array(canvasHeight);
    }
    points[canvasWidth-1][canvasHeight-1] = 0;
}

function drawPoints(x0, y0, x1, y1){
    var dx = Math.abs(x1-x0);
    var dy = Math.abs(y1-y0);
    var sx = (x0 < x1) ? 1 : -1;
    var sy = (y0 < y1) ? 1 : -1;
    var err = dx-dy;

    while(true){
        points[x0][y0] = 1;
        ctx.fillRect(x0, y0, 1, 1);
        if (Math.abs(x0-x1)<0.0001 && Math.abs(y0-y1)<0.0001) break;
        var e2 = 2*err;
        if (e2 >-dy){ err -= dy; x0  += sx; }
        if (e2 < dx){ err += dx; y0  += sy; }
    }
}
function getPoints(){
    return points;
}
function drawSolution(array){
    ctx.fillStyle = "#fdc855";
    for(var i = 0; i < array.length-1; i++){
        ctx.fillRect(array[i].x, array[i].y, 1, 1);
    }
}

function send(h, w, solution, _points){
    var distance = 0;
    var totalDistance = 0;
    var tempD = 0;
    var tempPoints = [];

    for(var x = 0; x < w-1; x++){
        for(var y = 0; y < h-1; y++){
            if(_points[x][y] === 1) {
                tempPoints.push({x: x, y: y})
            }
        }
    }
    for(var i = 0; i < tempPoints.length-1; i++){
        distance = Math.sqrt(Math.pow(solution[0].x - (tempPoints[i].x), 2) + Math.pow(solution[0].y - (tempPoints[i].y), 2));
        for(var j = 1; j < solution.length-1; j++){
            tempD = Math.sqrt(Math.pow(solution[j].x - (tempPoints[i].x), 2) + Math.pow(solution[j].y - (tempPoints[i].y), 2));
            if(tempD < distance) distance = tempD;
        }
        totalDistance += distance;
    }
    return (totalDistance/tempPoints.length);
}

WorkerScript.onMessage = function(message) {
    var calculatedResult = send(message.height, message.width, message.solution, message.points);
    WorkerScript.sendMessage({result: calculatedResult});
}
