import QtQuick 2.0
import ImageLoader 1.0
import QtQuick.Window 2.2
import QtQuick.Dialogs 1.1
import Enginio 1.0
import "landmarksGold.js" as JS
import "../shared"
import "../images"

Rectangle {
    id: screen
    color: "black"

    property int buttonSize: screen.height/10
    property bool noError: true

    ImageLoader{
        id: loader
    }
    EnginioClient{
        id: enginio
        backendId: "550aa0cfc8e85c1504026a9e"
        onFinished: if(noError){
                        animation.visible = false;
                        confirmDialog.visible = true
                    }
        onError: {
            noError = false;
            errorDialog.visible = true;
        }
    }
    Flickable {
        id: flick
        width: Screen.width
        height: Screen.height
        anchors.fill: parent
        contentWidth: image.width
        contentHeight: image.height + buttonSize

        Image {
            id:image
            Component.onCompleted: {
                openFileButton.active = true
                var filename = loader.openFile()
                if(filename) {
                    image.source = "file:///" + filename;
                    JS.setUpCanvas(canvas, flick.width/20);
                }
                openFileButton.active = false
            }

            Canvas {
                id:canvas
                anchors.fill: parent
                onPaint: {
                    if(undoButton.active) JS.remove()
                    else JS.paintLine()
                }

                MouseArea{
                    anchors.fill: parent
                    onClicked:{
                        if(mmButton.active){
                            offset.getImage().visible = true;
                            offset.getImage().x = Math.floor(mouseX) - (offset.getImage().width/2);
                            offset.getImage().y = Math.floor(mouseY) - (offset.getImage().height/2);
                            offset.active = offset.active ? false : true
                        }
                    }
                    onPositionChanged:
                        if(!flick.interactive){
                            JS.setPosition(Math.floor(mouseX), Math.floor(mouseY))
                            canvas.requestPaint()
                        }
                    onReleased:{
                        JS.resetPosition()
                    }
                }
            }
        }
        Offset{
            id:offset
            anchors.fill: parent
            imageSize: buttonSize
        }
    }
    MessageDialog {
        id: errorDialog
        text: "Could not connect to the cloud database.\n\nAn internet connection is required to upload new exercises."
        Component.onCompleted: visible = false;
    }
    MessageDialog {
        id: exitDialog
        text: "Do you want to exit to main menu?"
        standardButtons: StandardButton.No | StandardButton.Yes
        onYes: state.stateUrl = Qt.resolvedUrl("loading.qml");
        onNo: exitButton.active = false;
        Component.onCompleted: visible = false
    }
    MessageDialog {
        id: confirmDialog
        text: "Case saved successfully"
        onAccepted: state.stateUrl = Qt.resolvedUrl("loading.qml")
        Component.onCompleted: visible = false
    }
    MessageDialog {
        id: noSolutionDialog
        text: "No solution detected.\n\nPlease use the pencil tool to draw a border."
        onAccepted: animation.visible = false
        Component.onCompleted: visible = false
    }
    MessageDialog {
        id: sendDialog
        text: "Are you ready to save your solution?"
        standardButtons: StandardButton.No | StandardButton.Yes
        onYes:{
            animation.visible = true;
            if(JS.send().length < 1) noSolutionDialog.visible = true;
            else{
                var os = 0;
                if(parseInt(offsetText.text) !== 0 && offsetText.text.length > 1) os = offset.offset()/parseInt(offsetText.text);
                var pathParts = image.source.toString().split("/");
                var fileName = pathParts[pathParts.length - 1];
                var fileObject = {
                    objectType: "objects.level",
                    name: fileName,
                    gamemode: "tb",
                    solution: JS.solution,
                    localPath: image.source,
                    mm: os
                }
                var reply = enginio.create(fileObject);
                reply.finished.connect(function() {
                    var uploadData = {
                        file: { fileName: fileName },
                        targetFileProperty: {
                            objectType: "objects.level",
                            id: reply.data.id,
                            propertyName: "file"
                        },
                    };
                    enginio.uploadFile(uploadData, image.source);
                })
            }
        }
        onNo: sendButton.active = false;
        Component.onCompleted: visible = false

    }
    MessageDialog {
        id: clearDialog
        text: "Do you really want to clear your drawing?"
        standardButtons: StandardButton.No | StandardButton.Yes
        onYes:{
            undoButton.active = false
            pencilButton.active = false
            trashButton.active = false
            flick.interactive = true
            JS.clearCanvas(canvas.height, canvas.width)
            canvas.requestPaint()
        }
        onNo:  trashButton.active = false
        Component.onCompleted: visible = false

    }
    Image {
        id: layout
        width: screen.width
        height: buttonSize + 2
        anchors.left: parent.left
        anchors.bottom: parent.bottom
        source: "/images/images/bar.png"

        Row {
            anchors.bottom: parent.bottom
            anchors.left: parent.left
            anchors.right: parent.right

            Button {
                id: pencilButton
                width: buttonSize
                height: buttonSize
                anchors.bottom: parent.bottom
                imgSrc: active ? "/images/images/buttonPencilActive.png" : "/images/images/buttonPencil.png";
                onClicked: {
                    if(pencilButton.active){
                        flick.interactive = true
                        pencilButton.active = false
                    }
                    else{
                        flick.interactive = false
                        pencilButton.active = true

                        undoButton.active = false
                    }
                }
            }
            Button {
                id: undoButton
                width: buttonSize
                height: buttonSize
                anchors.bottom: parent.bottom
                imgSrc: active ? "/images/images/buttonUndoActive.png" : "/images/images/buttonUndo.png"
                onClicked: {
                    if(undoButton.active){
                        flick.interactive = true
                        undoButton.active = false
                    }
                    else{
                        flick.interactive = false
                        undoButton.active = true
                        pencilButton.active = false
                    }
                }
            }
            Button {
                id: trashButton
                width: buttonSize
                height: buttonSize
                anchors.bottom: parent.bottom
                imgSrc: active ? "/images/images/buttonTrashActive.png" : "/images/images/buttonTrash.png"
                onClicked: {
                    trashButton.active = true
                    clearDialog.visible = true
                }
            }
            Button{
                id: mmButton
                anchors.bottom: parent.bottom
                width: buttonSize
                height: buttonSize
                imgSrc: active ? "/images/images/buttonMMActive.png" : "/images/images/buttonMM.png"
                onClicked: {
                    if(mmButton.active){
                        Qt.inputMethod.hide();
                        mmButton.active = false;
                        offset.visible = false;
                        mmLayout.visible = false;
                    }
                    else{
                        mmButton.active = true;
                        undoButton.active = false
                        pencilButton.active = false
                        offset.visible = true;
                        mmLayout.visible = true;
                    }
                }
            }
            Button{
                id: hideButton
                anchors.bottom: parent.bottom
                width: buttonSize
                height: buttonSize
                imgSrc: active ? "/images/images/buttonEyeActive.png" : "/images/images/buttonEye.png"
                onClicked: {
                    if(hideButton.active){
                        canvas.visible = true;
                        hideButton.active = false;
                    }
                    else{
                        canvas.visible = false;
                        hideButton.active = true;
                    }
                }
            }
            Row{
                id: mmLayout
                visible: false
                anchors.verticalCenter: parent.verticalCenter

                Text {
                    text: "Offset:"
                    font.pointSize: buttonSize/2
                    anchors.verticalCenter: parent.verticalCenter
                }
                Rectangle{
                    color: "white"
                    border.color: "black"
                    height: (buttonSize/2) + 5
                    width: buttonSize * 2
                    anchors.verticalCenter: parent.verticalCenter

                    TextInput {
                        id: offsetText
                        anchors.right: parent.right
                        anchors.verticalCenter: parent.verticalCenter
                        text: "0"
                        font.pointSize: buttonSize/2
                        Keys.onEnterPressed: {
                            Qt.inputMethod.hide();
                        }
                        Keys.onReturnPressed: {
                            Qt.inputMethod.hide();
                        }
                    }
                    MouseArea{
                        anchors.fill: parent
                        onClicked: {
                            offsetText.focus = false;
                            offsetText.selectAll();
                            offsetText.forceActiveFocus();
                        }
                    }
                }
                Text {
                    text: "mm"
                    font.pointSize: buttonSize/2
                    anchors.verticalCenter: parent.verticalCenter
                }
            }
        }
        Row {
            layoutDirection: "RightToLeft"
            anchors.bottom: parent.bottom
            anchors.left: parent.left
            anchors.right: parent.right

            Button {
                id: sendButton
                imgSrc: active ? "/images/images/buttonSaveActive.png" : "/images/images/buttonSave.png"
                anchors.bottom: parent.bottom
                width: buttonSize
                height: buttonSize
                onClicked: {
                    sendButton.active = true
                    sendDialog.visible = true
                }
            }

            Button {
                id: openFileButton
                width: buttonSize
                height: buttonSize
                anchors.bottom: parent.bottom
                imgSrc: active ? "/images/images/buttonOpenFileActive.png" : "/images/images/buttonOpenFile.png"
                onClicked: {
                    openFileButton.active = true
                    var filename = loader.openFile()
                    if(filename) {
                        image.source = "file:///" + filename;
                        JS.setUpCanvas(canvas, flick.width/20);
                    }
                    openFileButton.active = false
                }
            }
        }
    }
    Button{
        id: exitButton
        anchors.right: parent.right
        width: buttonSize
        height: buttonSize
        imgSrc: active ? "/images/images/settingsButtonActive.png" : "/images/images/settingsButton.png"
        onClicked: {
            exitButton.active = true
            exitDialog.visible = true
        }
    }
    Image{
        id: animation
        anchors.horizontalCenter: screen.horizontalCenter
        anchors.verticalCenter: screen.verticalCenter
        visible: false;
        width: buttonSize*2
        height: buttonSize*2
        source: "/images/images/loadingIcon.png"

        RotationAnimation on rotation {
            loops: Animation.Infinite
            from: 0
            to: 360
        }
    }
    Item {
        id: state
        visible: false
        property url stateUrl
        onStateUrlChanged: visible = (stateUrl == '') ? false : true;
        anchors.fill: parent

        Loader{
            focus: true
            source: state.stateUrl
            anchors.fill: parent
        }
    }
}
